require 'net/http'
require 'open-uri'
require 'openssl'
require 'json'

module SptyAPI::HTTP
  class Client
    attr_accessor :path_parameter, :query_parameter, :body_parameter,
                  :http_method, :token_type, :access_token, :endpoint,
                  :api_overwrite, :content_type_overwrite

    DEFAULT_API   = 'https://api.spotify.com/v1/'
    CONTENT_TYPE  = 'application/json'

    def initialize(http_method:, token_type:, access_token: nil, endpoint:)
      self.http_method  = http_method
      self.token_type   = token_type
      self.access_token = access_token
      self.endpoint     = endpoint
    end

    def request
      uri = prepare_uri
      http = prepare_http(uri)
      http.request(prepare_request(uri))
    end

    private

    def prepare_uri
      id  = (path_parameter.nil?) ? '' : path_parameter[:id]
      api = (api_overwrite.nil?) ? DEFAULT_API : api_overwrite
      uri = "#{api}#{endpoint}"
      uri += "/#{id}" if id != ''
      uri += "?#{prepare_query_parameter}"
      URI.parse(uri)
    end

    def prepare_query_parameter
      return '' if query_parameter.nil?

      q_params = ''
      query_parameter.each_with_index do |(key, value), index|
        q_params += '&' if index > 0
        q_params += "#{key}=#{value}"
      end

      q_params
    end

    def prepare_http(uri)
      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE

      http
    end

    def prepare_request(uri)
      req = self.send("setup_request_#{http_method}", uri)
      req['Authorization'] = "#{token_type} #{access_token}"

      req
    end

    def prepare_content_type
      (content_type_overwrite.nil?) ? CONTENT_TYPE : content_type_overwrite
    end

    def prepare_body_parameter
      return body_parameter.to_json if content_type_overwrite.nil?
      body_parameter
    end

    def setup_request_get(uri)
      Net::HTTP::Get.new(uri.request_uri)
    end

    def setup_request_post(uri)
      req = Net::HTTP::Post.new(uri.request_uri)
      req.body = prepare_body_parameter
      req["Content-Type"] = prepare_content_type

      req
    end

    def setup_request_put(uri)
      req = Net::HTTP::Put.new(uri.request_uri)
      req.body = prepare_body_parameter
      req["Content-Type"] = prepare_content_type

      req
    end

    def setup_request_delete(uri)
      Net::HTTP::Delete.new(uri.request_uri)
    end
  end
end
